import set from 'lodash/set';
import { Interpolation } from '@cloudflare/intl-types';

/**
 * Prepares a catalog for use with our intl libraries and exports
 * a keys object, which can be used to get the final key of a string:
 *
 * @example
 * import { prepareTranslations } from '@cloudflare/intl-core'
 * import { Trans } from '@cloudflare/intl-react'
 * const { keys, translations } = prepareTranslations('my-product', {
 *   someFeature: {
 *     title: 'Some feature title',
 *     description: 'Some feature description',
 *   },
 *   help: 'The help text'
 * })
 *
 * console.log(keys.someFeature.title) // "my-product.someFeature.title"
 *
 * <Trans id={keys.someFeature.description} />
 *
 * @param namespace Namespace of the catalog (prefixes with "${namespace}.")
 * @param trans The unprefixed keys/vals of the english catalog
 */
export function prepareTranslations(namespace, trans) {
  var translations = flattenCatalog(namespace, trans);
  var keys = Object.keys(translations).reduce((result, key) => set(result, key.substring("".concat(namespace, ".").length), key), {});
  return {
    translations,
    keys,
    namespace
  };
}

/**
 * Takes a nested catalog of the form:
 * {
 *   feature: {
 *     modal: {
 *       title: "Hello!"
 *     }
 *   }
 * }
 *
 * And flattens it, prefixing a namespace:
 *
 * {
 *   "feature.modal.title": "Hello!"
 * }
 * @param namespace Namespace of the catalog
 * @param obj The translations catalog
 */
function flattenCatalog(namespace, obj) {
  var result = {};
  var keys = [namespace];
  function iterate(obj) {
    Object.keys(obj).forEach(key => {
      keys.push(key);
      if (typeof obj[key] === 'string' || obj[key] instanceof Interpolation) {
        result[keys.join('.')] = obj[key].toString();
      } else if (typeof obj[key] === 'object' && obj[key] !== null) {
        iterate(obj[key]);
      }
      keys.pop();
    });
  }
  iterate(obj);
  return result;
}