"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.prepareTranslations = prepareTranslations;
var _set = _interopRequireDefault(require("lodash/set"));
var _intlTypes = require("@cloudflare/intl-types");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
/**
 * Prepares a catalog for use with our intl libraries and exports
 * a keys object, which can be used to get the final key of a string:
 *
 * @example
 * import { prepareTranslations } from '@cloudflare/intl-core'
 * import { Trans } from '@cloudflare/intl-react'
 * const { keys, translations } = prepareTranslations('my-product', {
 *   someFeature: {
 *     title: 'Some feature title',
 *     description: 'Some feature description',
 *   },
 *   help: 'The help text'
 * })
 *
 * console.log(keys.someFeature.title) // "my-product.someFeature.title"
 *
 * <Trans id={keys.someFeature.description} />
 *
 * @param namespace Namespace of the catalog (prefixes with "${namespace}.")
 * @param trans The unprefixed keys/vals of the english catalog
 */
function prepareTranslations(namespace, trans) {
  var translations = flattenCatalog(namespace, trans);
  var keys = Object.keys(translations).reduce(function (result, key) {
    return (0, _set.default)(result, key.substring("".concat(namespace, ".").length), key);
  }, {});
  return {
    translations: translations,
    keys: keys,
    namespace: namespace
  };
}

/**
 * Takes a nested catalog of the form:
 * {
 *   feature: {
 *     modal: {
 *       title: "Hello!"
 *     }
 *   }
 * }
 *
 * And flattens it, prefixing a namespace:
 *
 * {
 *   "feature.modal.title": "Hello!"
 * }
 * @param namespace Namespace of the catalog
 * @param obj The translations catalog
 */
function flattenCatalog(namespace, obj) {
  var result = {};
  var keys = [namespace];
  function iterate(obj) {
    Object.keys(obj).forEach(function (key) {
      keys.push(key);
      if (typeof obj[key] === 'string' || obj[key] instanceof _intlTypes.Interpolation) {
        result[keys.join('.')] = obj[key].toString();
      } else if (_typeof(obj[key]) === 'object' && obj[key] !== null) {
        iterate(obj[key]);
      }
      keys.pop();
    });
  }
  iterate(obj);
  return result;
}